package org.openxma.dsl.reference.dao.impl;

import org.openxma.dsl.platform.dao.*;
import org.openxma.dsl.platform.dao.impl.*;
import org.openxma.dsl.platform.service.Mapper;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import org.openxma.dsl.reference.dao.CategoryDao;
import org.openxma.dsl.reference.model.Category;
import org.openxma.dsl.reference.model.impl.CategoryImpl;
/**
 * @see CategoryDao
 */
@SuppressWarnings("unused")
public abstract class CategoryDaoGenImpl extends GenericDaoHibernateImpl<Category,Long> implements CategoryDao {

    public CategoryDaoGenImpl() {
        super(CategoryImpl.class);
    }
    
    /**
     * @param context the context for the object creation
     * @return a transient instance of entity <code>Category</code> with an already assigned identifier.
     */
    public Category createEntity(Object context) {
        return create();
    }

    public Category create()
    {
        Category category = createEntityInstance();
        category.setOid(Long.valueOf(dataFieldMaxValueIncrementer.nextLongValue()));
        return category;
    }

    public Category create(String name) {
        Assert.notNull(name, "Parameter 'name' must not be null");
        Category category = (Category) create();
        category.setName(name);
        return category;
    }

    public Category createAndSave(String name) {
        Category category = create(name);
        saveOrUpdate(category);
        return category;
    }
    
    protected Mapper getNamedQueryMapper(String queryName) {
		return new CategoryDaoQueryMapper();
	}
	
	protected class CategoryDaoQueryMapper extends NamedQueryMapper {
	
		@SuppressWarnings("unchecked")
		public <E> E mapOne(Object sourceObject, E targetObject, Object context) {
			return super.mapOne(sourceObject, targetObject);
		}
	}
}
