package org.openxma.dsl.reference.dtoassembler.dao.impl;

import org.openxma.dsl.platform.dao.*;
import org.openxma.dsl.platform.dao.impl.*;
import org.openxma.dsl.platform.service.Mapper;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.jdbc.support.incrementer.DataFieldMaxValueIncrementer;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;
import org.openxma.dsl.reference.dtoassembler.dao.ChildDao;
import org.openxma.dsl.reference.dtoassembler.model.Child;
import org.openxma.dsl.reference.dtoassembler.model.Parent;
import org.openxma.dsl.reference.dtoassembler.model.impl.ChildImpl;
/**
 * @see ChildDao
 */
@SuppressWarnings("unused")
public abstract class ChildDaoGenImpl extends GenericDaoHibernateImpl<Child,String> implements ChildDao {

    public ChildDaoGenImpl() {
        super(ChildImpl.class);
    }
    
    /**
     * @param context the context for the object creation
     * @return a transient instance of entity <code>Child</code> with an already assigned identifier.
     */
    public Child createEntity(Object context) {
        return create();
    }

    public Child create()
    {
        Child child = createEntityInstance();
        child.setOid(dataFieldMaxValueIncrementer.nextStringValue());
        return child;
    }

    public Child create(Parent parent) {
        Assert.notNull(parent, "Parameter 'parent' must not be null");
        Child child = (Child) create();
        child.setParent(parent);
        return child;
    }

    public Child createAndSave(Parent parent) {
        Child child = create(parent);
        saveOrUpdate(child);
        return child;
    }
    
    protected Mapper getNamedQueryMapper(String queryName) {
		return new ChildDaoQueryMapper();
	}
	
	protected class ChildDaoQueryMapper extends NamedQueryMapper {
	
		@SuppressWarnings("unchecked")
		public <E> E mapOne(Object sourceObject, E targetObject, Object context) {
			return super.mapOne(sourceObject, targetObject);
		}
	}
}
