package org.openxma.dsl.reference.service.impl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.openxma.dsl.platform.service.Mapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.openxma.dsl.reference.dao.AddressDao;
import org.openxma.dsl.reference.dao.OrderDao;
import org.openxma.dsl.reference.dao.PrivateCustomerDao;
import org.openxma.dsl.reference.dto.PrivateCustomerView;
import org.openxma.dsl.reference.model.Address;
import org.openxma.dsl.reference.model.PrivateCustomer;
import org.openxma.dsl.reference.service.PrivateCustomerService;
import org.springframework.dao.OptimisticLockingFailureException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
public abstract class PrivateCustomerServiceGenImpl implements PrivateCustomerService {
    protected final Log logger = LogFactory.getLog(getClass());
    @Autowired(required=false)
    protected Mapper mapper;
    protected PrivateCustomerDao privateCustomerDao;
    protected AddressDao addressDao;
    protected OrderDao orderDao;
    
    @Autowired
    public void setPrivateCustomerDao(PrivateCustomerDao privateCustomerDao) {
        this.privateCustomerDao = privateCustomerDao;
    }

    @Autowired
    public void setAddressDao(AddressDao addressDao) {
        this.addressDao = addressDao;
    }

    @Autowired
    public void setOrderDao(OrderDao orderDao) {
        this.orderDao = orderDao;
    }
    
    @Transactional
	public PrivateCustomerView save(PrivateCustomerView privateCustomerView) {
        Assert.notNull(privateCustomerView, "parameter 'privateCustomerView' must not be null");
        PrivateCustomer privateCustomer = this.mapper.createAndMapOne(privateCustomerView, PrivateCustomer.class,"savePrivateCustomer");
        Address invoiceAddress = this.mapper.createAndMapOne(privateCustomerView, Address.class,"invoiceAddress");
        invoiceAddress = addressDao.load(invoiceAddress.getOid());
        privateCustomer.setInvoiceAddress(invoiceAddress);
        Address deliveryAddress = this.mapper.createAndMapOne(privateCustomerView, Address.class,"deliveryAddress");
        deliveryAddress = addressDao.load(deliveryAddress.getOid());
        privateCustomer.setDeliveryAddress(deliveryAddress);
        this.privateCustomerDao.saveOrUpdate(privateCustomer);
        return this.mapper.mapOne(privateCustomer,new PrivateCustomerView());
    }
    
    @Transactional(readOnly = true)
	public PrivateCustomerView loadPrivateCustomerView(String oid) {
        Assert.notNull(oid, "parameter 'oid' must not be null");
        return this.mapper.createAndMapOne(this.privateCustomerDao.load(oid),PrivateCustomerView.class);
    }
    
    @Transactional
	public void update(PrivateCustomerView privateCustomerView) {
        Assert.notNull(privateCustomerView, "parameter 'privateCustomerView' must not be null");
        PrivateCustomer mappedPrivateCustomer = this.mapper.createAndMapOne(privateCustomerView, PrivateCustomer.class);
        PrivateCustomer privateCustomer = this.privateCustomerDao.load(mappedPrivateCustomer.getOid());
        if (!privateCustomer.getVersion().equals(mappedPrivateCustomer.getVersion())) {
            throw new OptimisticLockingFailureException("Entity 'PrivateCustomer' was updated or deleted by another transaction.");
        }
        this.mapper.mapOne(privateCustomerView, privateCustomer,"updatePrivateCustomer");
    }
    
    @Transactional
	public void deletePrivateCustomer(String oid) {
        Assert.notNull(oid, "parameter 'oid' must not be null");
        this.privateCustomerDao.delete(oid);
    }
        
    @Transactional
    public PrivateCustomerView saveOrUpdate(PrivateCustomerView privateCustomerView) {
        Assert.notNull(privateCustomerView, "parameter 'privateCustomerView' must not be null");
        if ( privateCustomerView.getVersion() == null ) {
            return this.save(privateCustomerView);
        }
        else {
            this.update(privateCustomerView);
            return privateCustomerView;
        }
    }

    
}
